/* configureDialog.h

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#ifndef CONFIGUREDIALOG_H
#define CONFIGUREDIALOG_H

#include "palomaDialog.h"
#include "form.h"
#include "dataset.h"
#include "pickOne.h"
#include "tableData.h"

#include <gtk--.h>
#include <map>

class InfoMessage;

class ConfigureDialog : public PalomaDialog {
public:
  ConfigureDialog(const Dataset &data, int form_id);
  ~ConfigureDialog();

  virtual void record_db();
  void record_sfdef();
  void record_convert();
  void record_extensions();
  void record_vmdef();

private:
  void new_soundfile_clicked();
  void delete_soundfile_clicked();
  void delete_soundfile_okayed(InfoMessage *dialog, int soundfile_id);
  void new_volume_clicked();
  void delete_volume_clicked();
  void delete_volume_okayed(InfoMessage *dialog, int volume_id);
  void ok_button_clicked();
  void cancel_button_clicked();

  void layout_window();
  void layout_cdrom_page();
  void layout_soundfile_page();
  void layout_volume_page();
  
  void new_sfdef();
  void save_sfdef();
  void new_sfconvert();
  void save_sfconvert();
  void new_vmdef();
  void save_vmdef();

  Gtk_ItemFactory_MenuBar _menu;
  Gtk_ObjectHandle<Gtk_MenuBar> _menubar;
  Gtk_VBox _box1;
  Gtk_Notebook _notebook;

  Gtk_Label _cdrom_tab;
  Gtk_VBox _cdrom_box;
  Gtk_Label _cd_cdrom_type_label;
  PickOne _cd_cdrom_type_pick;
  Gtk_Label _cd_cdrom_load_type_label;
  PickOne _cd_cdrom_load_type_pick;
  Gtk_Label _cd_cdrom_want_type_label;
  PickOne _cd_cdrom_want_type_pick;
  Gtk_Label _cd_cdrom_load_volume_label;
  PickOne _cd_cdrom_load_volume_pick;
  Gtk_Label _cd_cdrom_want_volume_label;
  PickOne _cd_cdrom_want_volume_pick;
  Gtk_Label _cd_sfcomment_label;

  Gtk_Label _soundfile_tab;
  Gtk_VBox _soundfile_box;
  Gtk_Label _soundfile_label;
  PickOne _soundfile_pick;
  Gtk_Button _new_soundfile;
  Gtk_Button _delete_soundfile;
  Gtk_HBox _sf_cbox;
  Gtk_Label _sf_play_command_label;
  Gtk_Entry _sf_play_command_entry;
  Gtk_Label _sf_play_response_label;
  Gtk_Entry _sf_play_response_entry;
  Gtk_Label _sf_verify_command_label;
  Gtk_Entry _sf_verify_command_entry;
  Gtk_Label _sf_verify_response_label;
  Gtk_Entry _sf_verify_response_entry;
  Gtk_HBox _sf_convert_box;
  Gtk_Label _sf_convert_label;
  PickOne _sf_convert_pick;
  Gtk_Entry _sf_convert_entry;
  Gtk_Label _sf_extensions_label;
  Gtk_Entry _sf_extensions_entry;

  Gtk_Label _volume_tab;
  Gtk_VBox _volume_box;
  Gtk_Label _volume_label;
  PickOne _volume_pick;
  Gtk_Button _new_volume;
  Gtk_Button _delete_volume;
  Gtk_HBox _vm_cbox;
  Gtk_Label _vm_definition_label;
  Gtk_Entry _vm_definition_entry;
  Gtk_Label _vm_bsize_label;
  Gtk_Entry _vm_bsize_entry;
  Gtk_Label _vm_bcurrent_label;
  Gtk_Entry _vm_bcurrent_entry;
  Gtk_Label _vm_bname_label;
  Gtk_Entry _vm_bname_entry;

  Gtk_Button _ok_button;
  Gtk_Button _cancel_button;
  int _form_id;

  typedef map<int, TableDataProxy> SFDefs;
  SFDefs _sfdefs;
  int _current_sfdef;

  // These are the sfdef entries which we have actually removed from
  // the database during the lifetime of this dialog.
  set<int> _removed_sfdefs;

  // This becomes a 2-d mapping of int x int -> TableData::Value.
  // Specifically, it maps from_type x to_type -> convert_command.
  typedef map<int, TableData::Value> SFConvertsInternal;
  typedef map<int, SFConvertsInternal> SFConverts;
  SFConverts _sfconverts;

  // And this maps from type -> extension_list.
  typedef map<int, TableData::Value> SFExtensions;
  SFExtensions _sfextensions;

  int _current_sfconvert;

  typedef map<int, TableDataProxy> VMDefs;
  VMDefs _vmdefs;
  int _current_vmdef;

  // For each vmdef flagged for deletion, we store a bool flag here
  // that indicates whether we are to also remove the songs attached
  // to this volume.
  typedef map<int, bool> VMDeleteSongs;
  VMDeleteSongs _vmdelete_songs;

  // These are the vmdef entries which we have actually removed from
  // the database during the lifetime of this dialog.
  set<int> _removed_vmdefs;
};

#endif

